/******************************************************************************
 File:    dpram.h
 Purpose: Header file for sample C language programs for the HOSTESS i.
          Contains dual port RAM address defines and data structures.
 Company: Comtrol Corporation
 Author:  Craig Harrison
*******************************************************************************
 Copyright 1991-1993 Comtrol Corporation. All rights reserved.
 Subject to developers license agreement.
******************************************************************************/


/* ----- Base address definitions ------------------------------------------ */
/* Far memory addresses are given here in segment:offset format, for example:
   0xd0005000 = d000:5000 */

#define BASE_ADDR_SYS     0xd0000000 /* SYS uP dual port RAM base address */
#define COMQ_ADDR_SYS     0xd0005000 /* SYS uP start addr of comq message q */

#if defined HOSTESSi || defined HOSTESS186
   #define IO_SYS 0x218              /* SYS uP base I/O address */
   #define ENTRY_SEG 0xc0            /* control program entry point segment */
   #define ENTRY_OFF 0               /* control program entry point offset */
   #define FIRMUSER_ADDR_SYS 0xd0000b80  /* SYS uP firmware user area addr */
   #define LOAD_ADDR_SYS 0xd0000c00  /* SYS uP ctrlpgm download address */
   #define FIRMUSER_ADDR 0x00000b80  /* COM uP firmware user area addr */
   #define COMQ_ADDR     0x00005000  /* COM uP start addr of comq message q */
#else if defined SMARTH
   #define IO_SYS        0x238       /* SYS uP base I/O address */
   #define ENTRY_SEG 0xc0            /* control program entry point segment */
   #define ENTRY_OFF 0x0             /* control program entry point offset */
   #define FIRMUSER_ADDR_SYS 0xd0000000  /* SYS uP firmware user area addr */
   #define LOAD_ADDR_SYS 0xd0000080  /* SYS uP ctrlpgm download address */
   #define LOAD_SEG      0x4000      /* COM uP source buffer segment */
   #define LOAD_OFF      0x80        /* COM uP source buffer offset */
   #define FIRMUSER_ADDR 0x40000000  /* COM uP firmware user area addr */
   #define COMQ_ADDR     0x40005000  /* COM uP start addr of comq message queue */
#endif

#if defined HOSTESSi
   #define NUMLINES      16          /* number of serial lines */
#else if defined HOSTESS186 || defined SMARTH
   #define NUMLINES      8           /* number of serial lines */
#endif

#define TXB_SIZE      512            /* transmit buffer size */
#define RXB_SIZE      2048           /* receive buffer size */


/* ----- Message Queue data structure ------------------------------------- */
/* Message format:
   Byte 0 = Command
   Byte 1 = Line number
   Byte 2-15 = Parameters as defined by command function
*/

#define MSG_LEN   16                   /* number of bytes in one message */
#define MSGQ_SIZE 32                   /* number messages in queue buffer */
struct msgq
{
   int head;                           /* queue head */
   int tail;                           /* queue tail */
   char buf[MSGQ_SIZE][MSG_LEN];       /* queue buffer */
};
typedef struct msgq MSGQ_T;


/* ----- Line Table Entry data structure ---------------------------------- */

struct line_entry
{
   int io_base;                        /* SCC base I/O address */
   unsigned line_status;               /* line status (defined below) */

   /* SCC register values */
   unsigned char WR2_;                 /* write register 2 */
   unsigned char WR3_;                 /* write register 3 */
   unsigned char WR4_;                 /* write register 4 */
   unsigned char WR5_;                 /* write register 5 */
   unsigned char WR12_;                /* write register 12 */
   unsigned char WR13_;                /* write register 13 */
   unsigned char WR15_;                /* write register 15 */
   unsigned char fill1;                /* filler for word allignment */

   /* Transmit queue info */
   int Txq_head;                       /* add data here */
   int Txq_tail;                       /* remove data here */
   char far *Txq_com;                  /* ptr to Tx buffer, COM uP address */
   char far *Txq_sys;                  /* ptr to Tx buffer, SYS uP address */

   /* Receive queue info */
   int Rxq_head;                       /* add data here */
   int Rxq_tail;                       /* remove data here */
   char far *Rxq_com;                  /* ptr to Tx buffer, COM uP address */
   char far *Rxq_sys;                  /* ptr to Tx buffer, SYS uP address */

   unsigned char fill2[12];            /* filler */
};
typedef struct line_entry LINE_ENTRY_T;


/* ----- line_status definitions ------------------------------------------ */
#define LINE_ACTIVE  0x0001   /* line is active */
#define TX_ACTIVE    0x0002   /* transmit is active (char is going out) */
#define RS485ENABLED 0x0004   /* set when using RS-485 on a line */
#define TXMODE       0x0008   /* RS-485 transceiver in Tx state */
#define ALLSENT_PEND 0x0010   /* waiting for all bit of last char to be sent */
#define SEND_FLAG    0x0020   /* next char to send is RS-485 flag char */
