/*****************************************************************************
File:    hilib.c
Purpose: C source for system processor routines to open, close, read from
         from, write to a serial line.  This module is compiled with another
         C language source module to form a DOS application program that runs
         on the system processor.  In the case of the sample programs diskette,
         HILIB.C and HITERM.C are compiled to form HITERM.EXE, a sample DOS
         application program that shows how the system processor interacts
         with the control program running on the controller board.
Company: Comtrol Corporation
Author:  Craig Harrison
Release: 1.0, 8-23-91 - Original release (Lori Slater, Craig Harrison)
         1.1, 6-26-92 - Craig Harrison
            Converted from assembly to C language.
            Added dpram_init() and BCOPY.
*******************************************************************************
Copyright 1991-1992 Comtrol Corporation. All rights reserved.
Subject to developers license agreement.
*******************************************************************************
                                 WARNING
The application program must call dpram_init() before calling any other function
in HILIB.C.
******************************************************************************/

#include <dos.h>
#include <mem.h>
#include "dpram.h"

/*****************************************************************************
Macro:    BCOPY
Purpose:  Copy a data buffer from one location in RAM to another.
Entry:    SRC: Ptr to source buffer
          DST: Ptr to destination buffer
          CNT: Number of bytes to write
Return:   Nothing
*/
#define BCOPY(SRC,DST,CNT)  movedata(FP_SEG((char far *)SRC), \
        FP_OFF((char far *)SRC),FP_SEG((char far *)DST), \
        FP_OFF((char far *)DST),(unsigned)(CNT))
/*****************************************************************************/

#define OUTB outportb

void dpram_init(void);
int enq_com_msg(char *);
int hiopen(int);
int hiclose(int);
int hiread(char *,int,int);
int hiwrite(char *,int,int);

static MSGQ_T far *comq;               /* ptr to comq message queue */
static MSGQ_T far *sysq;               /* ptr to sysq message queue */
static LINE_ENTRY_T far *line[16];     /* ptrs to line table entries */

/*****************************************************************************
Function: void dpram_init
Purpose:  Initialize pointers to dual port RAM structures.
Entry:    Nothing.
Return:   Nothing.
*/
void dpram_init(void)
{
   int linenum;                        /* line number */
   LINE_ENTRY_T far *lt_p;             /* ptr to line table entry */
   char far *ram_p;                    /* ptr to some RAM address */

   /* Initialize message queue pointers */
   ram_p = (char far *)COMQ_ADDR_SYS;  /* start addr of comq */
   comq = (MSGQ_T far *)ram_p;         /* ptr to comq */
   ram_p += sizeof(MSGQ_T);
   sysq = (MSGQ_T far *)ram_p;         /* ptr to sysq */
   ram_p += sizeof(MSGQ_T);
   ram_p += 8;    /* start line table on paragarph boundary for easy viewing */

   /* Initialize pointers to line table entries */
   for(linenum = 0;linenum < NUMLINES;linenum++,ram_p += sizeof(LINE_ENTRY_T))
      line[linenum] = (LINE_ENTRY_T far *)ram_p;
   ram_p += (16 - linenum) * sizeof(LINE_ENTRY_T); /* if < 16 lines must leave
                                empty space so Tx buffers properly addressed */

   /* Initialize SYSP Txq and Rxq buffer pointers in line table */
   for(linenum = 0;linenum < NUMLINES;linenum++,ram_p += TXB_SIZE)
      line[linenum]->Txq_sys = ram_p;
   ram_p += (16 - linenum) * TXB_SIZE;  /* if < 16 lines must leave empty space
                                           so Rx buffers properly addressed */
   for(linenum = 0;linenum < NUMLINES;linenum++,ram_p += RXB_SIZE)
      line[linenum]->Rxq_sys = ram_p;
}

/*****************************************************************************
Function: int enq_com_msg
Purpose:  Add a message to the Communications Processor queue.
Entry:    char *msgsrc: Source of message to enqueue, should be MSG_LEN
                        bytes long.
Return:   1 if message successfully enqueued, else 0.
*/
int enq_com_msg(char *msgsrc)
{
   char far *mb_p;                     /* ptr to message byte */
   int mi;                             /* message byte counter */
   int numopen;                        /* number open message spaces in queue */

   if((numopen = comq->tail - comq->head - 1) < 0) /* get open space in queue*/
      numopen += MSGQ_SIZE;            /* adjust for queue wrap */
   if(numopen <= 0)                    /* queue full */
      return(0);                       /* fail */

   mb_p = comq->buf[comq->head];       /* ptr into msg q buffer */
   for(mi = 0;mi < MSG_LEN;mi++)       /* copy message from source into q */
      *mb_p++ = *msgsrc++;
   comq->head = (comq->head + 1) % MSGQ_SIZE;   /* bump head pointer */
   return(1);                          /* success */
}

/*****************************************************************************
Function: void hiopen
Purpose:  Open a serial line.
Entry:    int linenum: Number of line to open (0-15)
Return:   1 if successful, 0 if failed
*/
int hiopen(int linenum)
{
   /* Default open message to controller */
   static char openmsg[MSG_LEN]=
   {
      1,             /* command number parameter */
      0,             /* line number paramters */
      0xc0,          /* WR3 parameters (Rx character size) */
      0x44,          /* WR4 parameters (stop bits, parity */
      0x60,          /* WR5 parameters   (Tx character size) */
      0x0e,          /* WR12 parameter   (lower byte of BRGTC) */
      0,             /* WR13 parameter   (upper byte of BRGTC) */
      0,             /* RS-485 parameter (0=disable RS485, 1=enable RS485) */
      0,0,0,0,0,0,0,0 /* unused */
   };

   openmsg[1] = (char)linenum;         /* set up line number in message */
   if(enq_com_msg(openmsg))            /* add message to COMQ */
   {
      OUTB(IO_SYS+2,0);                /* interrupt SYS uP */
      return(1);                       /* success */
   }
   else
      return(0);                       /* fail */
}

/*****************************************************************************
Function: void hiclose
Purpose:  Close a serial line.
Entry:    int linenum: Number of line to open (0-15)
Return:   1 if successful, 0 if failed
*/
int hiclose(int linenum)
{
   /* Default close message to controller */
   static char closemsg[MSG_LEN] =
   {
      2,                               /* command number parameter */
      0,                               /* line number parameter */
      0,0,0,0,0,0,0,0,0,0,0,0,0,0      /* unused */
   };

   closemsg[1] = (char)linenum;        /* set up line number in message */
   if(enq_com_msg(closemsg))           /* add message to COMQ */
   {
      OUTB(IO_SYS+2,0);                /* interrupt SYS uP */
      return(1);                       /* success */
   }
   else
      return(0);                       /* fail */
}

/*****************************************************************************
Function: int hiread
Purpose:  Read bytes from an open serial line on the controller.
Entry:    char far *sbuf: Ptr to SYS uP storage buffer to read into
          int cnt: Maximum number of bytes to read
          int linenum: Number (0-15) of serial line
Return:   Number of bytes read
*/
int hiread(char *sbuf,int cnt,int linenum)
{
   LINE_ENTRY_T far *lt_p;          /* ptr to line table entry */
   int i;                           /* balance of chars to copy after q wrap */
   int tail, head;                  /* Rx buffer head & tail ptrs */
   int heads;                       /* save copy of head ptr */
   int ccnt;                        /* count of chars copied */

   lt_p = line[linenum];               /* get ptr to line table entry */
   tail = lt_p->Rxq_tail;              /* read tail */
   head = lt_p->Rxq_head;              /* read head */
   heads = lt_p->Rxq_head;             /* read head again */

   /* Verify that head values match to prevent possibility that ctrlpgm
      modified it in between 8 bit SYS uP reads.  Only need to do this
      if controller is in 8 bit mode. */
   while(head != heads)                /* while the two head reads differ */
   {
      head = lt_p->Rxq_head;           /* read head */
      heads = lt_p->Rxq_head;          /* read head again */
   }

   /* Get number of characters in Rx buffer */
   if((ccnt = head - tail) < 0)
      ccnt += RXB_SIZE;                /* adjust for queue wrap */
   else if(ccnt == 0)
      return(ccnt);                    /* nothing in Rx buffer */
   if(cnt < ccnt)
      ccnt = cnt;                      /* don't overflow SYS uP buffer */
   i = ccnt - (RXB_SIZE - tail);       /* i = whats left after wrap around */
   if(i < 0)
      i = 0;

   /* Copy to end of Rx buffer */
   BCOPY(lt_p->Rxq_sys + tail,sbuf,ccnt - i);

   /* Point to beginning of buffer if already at end of it */
   tail = (tail + (ccnt - i)) % RXB_SIZE;

   /* Copy the rest of the buffer, if any left */
   if (i != 0)
   {
      BCOPY(lt_p->Rxq_sys,sbuf + (ccnt - i),i);
      tail = i;
   }

   /* Update Rx buffer tail */
   lt_p->Rxq_tail = tail;
   return(ccnt);
}

/*****************************************************************************
Function: int hiwrite
Purpose:  Write bytes to an open serial line on the controller.
Entry:    char far *sbuf: Ptr to SYS uP storage buffer to write from
          int cnt: Number of bytes to write
          int linenum: Number (0-15) of serial line
Return:   Number of bytes written
*/
int hiwrite(char *sbuf,int cnt,int linenum)
{
   LINE_ENTRY_T far *lt_p;          /* ptr to line table entry */
   int i;                           /* balance of chars to copy after q wrap */
   int numopen;                     /* num bytes open in Tx buffer */
   int head, tail;                  /* Tx buffer head & tail ptrs */
   int tails;                       /* save copy of tail ptr */

   lt_p = line[linenum];            /* get ptr to line table entry */
   head = lt_p->Txq_head;           /* read head */
   tail = lt_p->Txq_tail;           /* read tail */
   tails = lt_p->Txq_tail;          /* read head again */

   /* Verify that tail values match to prevent possibility that ctrlpgm
      modified it in between 8 bit SYS uP reads.  Only need to do this
      if controller is in 8 bit mode. */
   while(tail != tails)                /* while the two tail reads differ */
   {
      tail = lt_p->Txq_tail;           /* read tail */
      tails = lt_p->Txq_tail;          /* read tail again */
   }

   /* Get number bytes open in Tx buffer */
   if((numopen = tail - head - 1) < 0)
      numopen += TXB_SIZE;             /* adjust for q wrap */
   if(numopen > cnt)
      numopen = cnt;                   /* don't move more than are incoming */
   if(numopen == 0)
      return(0);                       /* no room in Tx buffer */

   i = numopen - (TXB_SIZE - head);    /* i = whats left after wrap around */
   if (i < 0)
      i = 0;

   /* Copy to end of Tx buffer */
   BCOPY(sbuf,lt_p->Txq_sys + head,numopen - i);

   /* Point to beginning of buffer if already at end of it */
   head = (head + (numopen - i)) % TXB_SIZE;

   /* Copy the rest of the buffer, if any left */
   if (i != 0)
   {
      BCOPY(sbuf + (numopen - i),lt_p->Txq_sys,i);
      head = i;
   }

   /* Update Tx buffer head */
   lt_p->Txq_head = head;
   return(numopen);
}
